import requests
import feedparser
from bs4 import BeautifulSoup
import os
import json
import re
from pathlib import Path
from dateutil import parser
import datetime

# Base URL and RSS feed URL
BASE_URL = "https://www.cityofmadison.com"
RSS_URL = "https://www.cityofmadison.com/police/newsroom/incidentreports/rss.cfm?a=71"

def fetch_rss_feed():
    """Fetch and parse the RSS feed."""
    response = requests.get(RSS_URL)
    if response.status_code != 200:
        print(f"Error fetching RSS feed: {response.status_code}")
        return None
    
    return feedparser.parse(response.content)

def extract_incident_data(html_content):
    """Extract incident data from HTML content."""
    soup = BeautifulSoup(html_content, 'html.parser')
    
    # Find the incident report details
    incident_report_div = soup.find('div', class_='incident-report-detail')
    if not incident_report_div:
        return None
    
    # Extract the case number from the title
    title_element = incident_report_div.find('h2', class_='title')
    case_number = None
    if title_element:
        case_match = re.search(r'#(\d+-\d+)', title_element.text)
        if case_match:
            case_number = case_match.group(1)
    
    # Extract the incident details
    incident_data = {}
    
    if case_number:
        incident_data['case_number'] = case_number
    
    # Find all rows with incident data
    detail_rows = soup.find_all('div', class_='clearfix')
    for row in detail_rows:
        # Look for key-value pairs
        key_element = row.find('strong')
        if key_element and key_element.text.strip():
            key = key_element.text.strip().rstrip(':')
            # Get the value by removing the key and extra whitespace
            value = row.get_text().replace(key_element.text, '').strip()
            if value:
                incident_data[key] = value
    
    # Extract the details section which might be in a different format
    details_section = soup.find('h3', string=lambda text: text and 'Details' in text)
    if details_section and details_section.find_next():
        details_text = details_section.find_next().get_text().strip()
        if details_text:
            incident_data['Details'] = details_text
    
    # If we couldn't find structured data, try to extract from general text
    if not incident_data and incident_report_div:
        # Extract all text and try to find patterns
        text = incident_report_div.get_text()
        
        # Try to extract common fields from text
        incident_type_match = re.search(r'Incident Type\s*(.+?)\s*Incident Date', text, re.DOTALL)
        if incident_type_match:
            incident_data['Incident Type'] = incident_type_match.group(1).strip()
        
        incident_date_match = re.search(r'Incident Date\s*(.+?)\s*Address', text, re.DOTALL)
        if incident_date_match:
            incident_data['Incident Date'] = incident_date_match.group(1).strip()
        
        address_match = re.search(r'Address\s*(.+?)\s*Update', text, re.DOTALL)
        if address_match:
            incident_data['Address'] = address_match.group(1).strip()
        
        update_match = re.search(r'Update\s*(.+?)\s*Details', text, re.DOTALL)
        if update_match:
            incident_data['Update'] = update_match.group(1).strip()
        
        details_match = re.search(r'Details\s*(.+?)$', text, re.DOTALL)
        if details_match:
            incident_data['Details'] = details_match.group(1).strip()
    
    return incident_data

def get_date_from_published(published_str):
    """Extract date from published string and format as YYYY-MM-DD."""
    try:
        # Parse the published date string
        date_obj = parser.parse(published_str)
        return date_obj.strftime('%Y-%m-%d')
    except Exception as e:
        print(f"Error parsing published date: {e}")
        # Fallback to current date
        return datetime.datetime.now().strftime('%Y-%m-%d')

def get_incident_file_path(incident_data):
    """Determine the file path for an incident based on its published date."""
    # Always use the published date for directory grouping
    if 'published' in incident_data:
        date_str = get_date_from_published(incident_data['published'])
    else:
        # Fallback to current date if no published date
        date_str = datetime.datetime.now().strftime('%Y-%m-%d')
    
    # Create safe case number for filename
    case_number = incident_data.get('case_number', 'unknown')
    safe_case_number = re.sub(r'[^\w\-]', '_', case_number)
    
    # Return path
    return Path(f"incidents/{date_str}/{safe_case_number}.json")

def process_incidents():
    """Process all incidents in the RSS feed."""
    feed = fetch_rss_feed()
    if not feed:
        return
    
    results = []
    
    for item in feed.entries:
        # Get the relative URL from the link
        relative_url = item.link.replace(BASE_URL, '')
        full_url = BASE_URL + relative_url
        
        try:
            # Try to extract case number from URL or title to check if we've already processed it
            case_match = re.search(r'incidentID=(\d+)', relative_url)
            incident_id = None
            if case_match:
                incident_id = case_match.group(1)
            
            # If we can't get the ID from URL, try the title
            if not incident_id and 'title' in item:
                case_match = re.search(r'#(\d+-\d+)', item.title)
                if case_match:
                    incident_id = case_match.group(1).replace('-', '_')
            
            # Use a temp incident data to check if file exists
            temp_incident_data = {
                'case_number': incident_id or 'temp',
                'published': item.published
            }
            
            file_path = get_incident_file_path(temp_incident_data)
            
            # Check if we've already processed this incident
            if file_path.exists():
                print(f"Skipping already processed incident: {file_path}")
                # Load existing data to include in results
                with open(file_path, 'r') as f:
                    existing_data = json.load(f)
                    results.append(existing_data)
                continue
            
            print(f"Processing: {full_url}")
            
            response = requests.get(full_url)
            if response.status_code == 200:
                incident_data = extract_incident_data(response.text)
                if incident_data:
                    # Add URL and publication date to the data
                    incident_data['url'] = full_url
                    incident_data['published'] = item.published
                    
                    results.append(incident_data)
                    print(f"Successfully processed incident: {incident_data.get('case_number', 'Unknown')}")
                else:
                    print(f"Could not extract data from {full_url}")
            else:
                print(f"Error fetching incident page: {response.status_code}")
        except Exception as e:
            print(f"Error processing {full_url}: {str(e)}")
    
    return results

def save_incidents(incidents):
    """Save incidents to JSON files organized by date."""
    if not incidents:
        print("No incidents to save")
        return
    
    # Group incidents by published date
    incidents_by_date = {}
    for incident in incidents:
        if 'published' in incident:
            date_str = get_date_from_published(incident['published'])
        else:
            date_str = datetime.datetime.now().strftime('%Y-%m-%d')
        
        if date_str not in incidents_by_date:
            incidents_by_date[date_str] = []
        
        incidents_by_date[date_str].append(incident)
    
    # Create directories and save incidents
    for date_str, date_incidents in incidents_by_date.items():
        # Create directory for this date
        date_dir = Path(f"incidents/{date_str}")
        date_dir.mkdir(parents=True, exist_ok=True)
        
        # Save all incidents for this date in one JSON file
        with open(date_dir / "incidents.json", 'w') as f:
            json.dump(date_incidents, f, indent=2)
        
        # Also save individual incident files
        for incident in date_incidents:
            case_number = incident.get('case_number', 'unknown')
            safe_case_number = re.sub(r'[^\w\-]', '_', case_number)
            individual_path = date_dir / f"{safe_case_number}.json"
            
            # Skip if file already exists (should be redundant with our earlier check, but just in case)
            if not individual_path.exists():
                with open(individual_path, 'w') as f:
                    json.dump(incident, f, indent=2)
        
        print(f"Saved {len(date_incidents)} incidents for {date_str}")

if __name__ == "__main__":
    print("Fetching Madison Police incident reports...")
    incidents = process_incidents()
    
    if incidents:
        print(f"Found {len(incidents)} incidents. Saving to disk...")
        save_incidents(incidents)
        print("Done!")
    else:
        print("No incidents found.")
